#!/bin/bash
# $Header: /home/hiro/projects/dvr2/scripts/RCS/ts2avi.sh,v 1.9 2013/02/11 21:22:02 hiro Exp $
#
# ts2avi.sh
#
#	== Project DVR2 ==
#
#	Script for decrypting and transcoding a TS file to an AVI file
#	Copyright (c) 2011 Hiro Sugawara
#
#	Usage: ts2avi.sh /path/to/yyyymmddhhmm-ccc-sss.ts

myname=`basename $0 .sh`
mydir=`dirname $0`

FFMPEG=/usr/local/bin/ffmpeg
FUSER=/usr/local/sbin/fuser

#
# Use remote BonCasServer
#
##export BONCASSERVER=mill2-vpn:36900

#
# Parse options
#
while [ $# -gt 0 ]
do
	case "$1" in
		-d*|-b*)
			dopt=$1
			shift
			;;
		-i*)
			iopt=$1
			shift
			;;
		-k*)
			kopt=$1
			shift
			;;
		*)
			break
			;;
	esac
done

#
# we must have a TS file in arguments
#
[ "$1" ]	|| {
	echo "$myname: TSFILE missing" >&2
	exit 1
}
TSFILE=$1

#
# File paths
#
IEPGFILE=`$mydir/dvr2-ts2iepg.pl $TSFILE`
[ "$IEPGFILE" ] || {
	echo "$myname: dvr2-ts2iepg.pl $TSFILE failed" >&2
	exit 1
}
FNAMECORE=`basename $IEPGFILE .iepg`
AVIDIR=`dirname $IEPGFILE`
AVIFILE="$AVIDIR/$FNAMECORE.avi"
ERRFILE="$AVIDIR/$FNAMECORE.err"
STATFILE="$AVIDIR/$FNAMECORE.stat"
TS2FILE="$AVIDIR/`basename $TSFILE`"

#
# FFMPEG parameters
#
VCODEC=mpeg4
##VTAG=DIVX
VTAG=XVID
VBITRATE=4000000
VSIZE=1280x720

#
# Delayed cleanup
#
RM() {
	mv $1 $1.bak
	[ "$kopt" ] || echo "rm -f $1.bak" | at now + 48 hours
}

B25='/usr/local/bin/b25 -s 1'
#
# Logging utilities
#
logit() {
	logger -p 6ga.info -t "$myname[$$]" $@
}

badtsname() {
	logit "bad TS file name $TSFILE ($1)"
	exit 1
}

#
# Utility to check if file is free to go
#
fileIsFree() {
	local file=$1
	local lsout=`ls -l $file`
	local lsout2

	for i in 1 2
	do
		if [ ! -s $file ]
		then
			logit "failed to find $file"
			exit 1
		fi

		if pid=`$FUSER $file`
		then
			logit "$file is busy (`ps $pid`)"
			exit 1
		fi
		sleep 2
		lsout2=`ls -l $file`
		if [ "$lsout" != "$lsout2" ]
		then
			logit "$file is busy (`ps $pid`)"
			exit 1
		fi
	done
}

#
# Decrypt into AVIDIR
#
decryptTS() {
	local tsfile=$1
	local ts2file=$2
	local statfile=$3

	fileIsFree $tsfile
	for i in 1 2 3
	do
		if pid=`$FUSER $tsfile 2>/dev/null`
		then
			logit "$tsfile is busy (`ps $pid`)"
			exit 1
		fi
		sleep 2
	done
	rm -f $ts2file

	logit "B25 decoding $tsfile to $ts2file"
	if [ "$BONCASSERVER" ]
	then
		logit "using BonCasServer at $BONCASSERVER"
	fi
	echo 'B25 decoding' >$statfile
	$mydir/dvr2-addiepg.pl $tsfile "avidir=$AVIDIR" || {
		echo "$myname: dvr2-addiepg.pl $tsfile 'avidir=$AVIDIR' failed" >&2
		exit 1
	}
	$B25 $tsfile $ts2file >/dev/null 2>&1
	if [ $? != 0 -o ! -s $tsfile ]
	then
		logit "B25 decryption of $tsfile failed"
		echo 'B25 failed' >$statfile
		rm -f $ts2file
		exit 1
	fi
	logit "B25 decryption of $tsfile completed"
	RM $tsfile
}

#
# Transcode with FFMPEG
#
transcode() {
	local ts2file=$1
	local avifile=$2
	local statfile=$3
	local errfile=$4

	logit "FFMPEG transcoding $ts2file to $avifile"
	fileIsFree $ts2file

	echo 'Transcoding' >$statfile
	$mydir/dvr2-addiepg.pl $ts2file	\
		"vcodec=$VCODEC" "vbitrate=$VBITRATE" "vsize=$VSIZE" || {
		echo "$myname: dvr2-addiepg.pl $tsfile 'vcodec=...' failed" >&2
		exit 1
	}
	$FFMPEG -y -i $TS2FILE -vcodec $VCODEC -vtag $VTAG -b:v $VBITRATE	\
		-s $VSIZE -map 0:0 -map 0:1 $avifile >>$errfile 2>&1

	if [ -s $avifile ]
	then
		logit "FFMPEG transcoding of $ts2file completed"
		echo 'Ready' >$statfile
		RM $ts2file
	else
		logit "FFMPEG transcoding of $ts2file failed"
		echo 'FFMPEG failed' >$statfile
		exit 1
	fi
}

#
# Main starts here
#

logit "starting processing $TSFILE" 
rm -f $STATFILE

if [ "$dopt" ]
then
	decryptTS $TSFILE $TS2FILE $STATFILE
fi

transcode $TS2FILE $AVIFILE $STATFILE $ERRFILE
